@@ -29,7 +29,7 @@ class QgsPolygon;
 /**
  * \ingroup core
  * \class QgsCurvePolygon
- * \brief Curve polygon geometry type
+ * \brief Curve polygon geometry type.
  */
 class CORE_EXPORT QgsCurvePolygon: public QgsSurface
 {
@@ -132,10 +132,15 @@ class CORE_EXPORT QgsCurvePolygon: public QgsSurface
     QString asKml( int precision = 17 ) const override;
     void normalize() final SIP_HOLDGIL;
 
+    /**
+     * Gets a polygon representation of this surface.
+     * Ownership is transferred to the caller.
+     */
+    virtual QgsPolygon *surfaceToPolygon() const SIP_FACTORY;
+
     //surface interface
     double area() const override SIP_HOLDGIL;
     double perimeter() const override SIP_HOLDGIL;
-    QgsPolygon *surfaceToPolygon() const override SIP_FACTORY;
     QgsAbstractGeometry *boundary() const override SIP_FACTORY;
     QgsCurvePolygon *snappedToGrid( double hSpacing, double vSpacing, double dSpacing = 0, double mSpacing = 0, bool removeRedundantPoints = false ) const override SIP_FACTORY;
     QgsCurvePolygon *simplifyByDistance( double tolerance ) const override SIP_FACTORY;
@@ -406,9 +411,11 @@ class CORE_EXPORT QgsCurvePolygon: public QgsSurface
      * Cast the \a geom to a QgsCurvePolygon.
      * Should be used by qgsgeometry_cast<QgsCurvePolygon *>( geometry ).
      *
-     * \note Not available in Python. Objects will be automatically be converted to the appropriate target type.
+     * Objects will be automatically converted to the appropriate target type.
+     *
+     * \note Not available in Python.
      */
-    inline static const QgsCurvePolygon *cast( const QgsAbstractGeometry *geom )
+    inline static const QgsCurvePolygon *cast( const QgsAbstractGeometry *geom ) // cppcheck-suppress duplInheritedMember
     {
       if ( !geom )
         return nullptr;
@@ -420,6 +427,27 @@ class CORE_EXPORT QgsCurvePolygon: public QgsSurface
         return static_cast<const QgsCurvePolygon *>( geom );
       return nullptr;
     }
+
+    /**
+     * Cast the \a geom to a QgsCurvePolygon.
+     * Should be used by qgsgeometry_cast<QgsCurvePolygon *>( geometry ).
+     *
+     * Objects will be automatically converted to the appropriate target type.
+     *
+     * \note Not available in Python.
+     */
+    inline static QgsCurvePolygon *cast( QgsAbstractGeometry *geom ) // cppcheck-suppress duplInheritedMember
+    {
+      if ( !geom )
+        return nullptr;
+
+      const Qgis::WkbType flatType = QgsWkbTypes::flatType( geom->wkbType() );
+      if ( flatType == Qgis::WkbType::CurvePolygon
+           || flatType == Qgis::WkbType::Polygon
+           || flatType == Qgis::WkbType::Triangle )
+        return static_cast<QgsCurvePolygon *>( geom );
+      return nullptr;
+    }
 #endif
 
     QgsCurvePolygon *createEmptyWithSameType() const override SIP_FACTORY;
