@@ -21,7 +21,6 @@
 #include "qgsidentifycontext.h"
 #include "qgsmaptool.h"
 #include "qgspointxy.h"
-#include "qgsunittypes.h"
 
 #include <QObject>
 #include <QPointer>
@@ -39,12 +38,13 @@ class QgsPointCloudLayer;
 class QgsPointCloudLayerElevationProperties;
 class QgsFeatureRenderer;
 class QgsExpressionContext;
+class QgsCoordinateReferenceSystem;
 
 /**
  * \ingroup gui
- * \brief Map tool for identifying features in layers
+ * \brief Map tool for identifying features in layers.
  *
- * after selecting a point, performs the identification:
+ * After the tool is used to select a point, this map tool performs the identification:
  *
  * - for raster layers shows value of underlying pixel
  * - for vector layers shows feature attributes within search radius
@@ -55,7 +55,6 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
     Q_OBJECT
 
   public:
-
     enum IdentifyMode
     {
       DefaultQgsSetting = -1,
@@ -66,11 +65,11 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
     };
     Q_ENUM( IdentifyMode )
 
-    enum Type
+    enum Type SIP_ENUM_BASETYPE( IntFlag )
     {
       VectorLayer = 1,
       RasterLayer = 2,
-      MeshLayer = 4, //!< \since QGIS 3.6
+      MeshLayer = 4,        //!< \since QGIS 3.6
       VectorTileLayer = 8,  //!< \since QGIS 3.14
       PointCloudLayer = 16, //!< \since QGIS 3.18
       AllLayers = VectorLayer | RasterLayer | MeshLayer | VectorTileLayer | PointCloudLayer
@@ -80,25 +79,32 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
 
     struct IdentifyResult
     {
-      //! Constructor for IdentifyResult
-      IdentifyResult() = default;
+        IdentifyResult() = default;
 
-      IdentifyResult( QgsMapLayer *layer, const QgsFeature &feature, const QMap< QString, QString > &derivedAttributes )
-        : mLayer( layer ), mFeature( feature ), mDerivedAttributes( derivedAttributes ) {}
+        IdentifyResult( QgsMapLayer *layer, const QgsFeature &feature, const QMap<QString, QString> &derivedAttributes )
+          : mLayer( layer ), mFeature( feature ), mDerivedAttributes( derivedAttributes ) {}
 
-      IdentifyResult( QgsMapLayer *layer, const QString &label, const QMap< QString, QString > &attributes, const QMap< QString, QString > &derivedAttributes )
-        : mLayer( layer ), mLabel( label ), mAttributes( attributes ), mDerivedAttributes( derivedAttributes ) {}
+        IdentifyResult( QgsMapLayer *layer, const QString &label, const QMap<QString, QString> &attributes, const QMap<QString, QString> &derivedAttributes )
+          : mLayer( layer ), mLabel( label ), mAttributes( attributes ), mDerivedAttributes( derivedAttributes ) {}
 
-      IdentifyResult( QgsMapLayer *layer, const QString &label, const QgsFields &fields, const QgsFeature &feature, const QMap< QString, QString > &derivedAttributes )
-        : mLayer( layer ), mLabel( label ), mFields( fields ), mFeature( feature ), mDerivedAttributes( derivedAttributes ) {}
+        IdentifyResult( QgsMapLayer *layer, const QString &label, const QgsFields &fields, const QgsFeature &feature, const QMap<QString, QString> &derivedAttributes )
+          : mLayer( layer ), mLabel( label ), mFields( fields ), mFeature( feature ), mDerivedAttributes( derivedAttributes ) {}
 
-      QgsMapLayer *mLayer = nullptr;
-      QString mLabel;
-      QgsFields mFields;
-      QgsFeature mFeature;
-      QMap< QString, QString > mAttributes;
-      QMap< QString, QString > mDerivedAttributes;
-      QMap< QString, QVariant > mParams;
+        QgsMapLayer *mLayer = nullptr;
+        QString mLabel;
+        QgsFields mFields;
+        QgsFeature mFeature;
+        QMap<QString, QString> mAttributes;
+        QMap<QString, QString> mDerivedAttributes;
+        QMap<QString, QVariant> mParams;
+    };
+
+    struct IdentifyProperties
+    {
+        //! Identify search radius is map units. Use negative value to ignore
+        double searchRadiusMapUnits = -1;
+        //! Skip identify results from layers that have a 3d renderer set
+        bool skip3DLayers = false;
     };
 
     //! constructor
@@ -168,12 +174,28 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
     void formatChanged( QgsRasterLayer *layer );
 
   signals:
-    void identifyProgress( int, int );
-    void identifyMessage( const QString & );
-    void changedRasterResults( QList<QgsMapToolIdentify::IdentifyResult> & );
+
+    /**
+     * Emitted when the identify action progresses.
+     *
+     * \param processed number of objects processed so far
+     * \param total total number of objects to process
+     */
+    void identifyProgress( int processed, int total );
+
+    /**
+     * Emitted when the identify operation needs to show a user-facing message
+     *
+     * \param message Message to show to the user
+     */
+    void identifyMessage( const QString &message );
+
+    /**
+     * Emitted when the format of raster \a results is changed and need to be updated in user-facing displays.
+     */
+    void changedRasterResults( QList<QgsMapToolIdentify::IdentifyResult> &results );
 
   protected:
-
     /**
      * Performs the identification.
      * To avoid being forced to specify IdentifyMode with a list of layers
@@ -186,7 +208,7 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
      * \param identifyContext Identify context object.
      * \returns a list of IdentifyResult
      */
-    QList<QgsMapToolIdentify::IdentifyResult> identify( int x, int y, IdentifyMode mode,  const QList<QgsMapLayer *> &layerList, LayerType layerType = AllLayers, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
+    QList<QgsMapToolIdentify::IdentifyResult> identify( int x, int y, IdentifyMode mode, const QList<QgsMapLayer *> &layerList, LayerType layerType = AllLayers, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
 
     QgsIdentifyMenu *mIdentifyMenu = nullptr;
 
@@ -222,35 +244,58 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
     bool identifyMeshLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsMeshLayer *layer, const QgsPointXY &point, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
 
     //! Returns derived attributes map for a clicked point in map coordinates. May be 2D or 3D point.
-    QMap< QString, QString > derivedAttributesForPoint( const QgsPoint &point );
+    QMap<QString, QString> derivedAttributesForPoint( const QgsPoint &point );
 
     /**
      * Overrides some map canvas properties inside the map tool for the upcoming identify requests.
      *
      * This is useful when the identification is triggered by some other piece of GUI like a 3D map view
      * and some properties like search radius need to be adjusted so that identification returns correct
-     * results. Currently only search radius may be overridden.
-     *
+     * results.
      * When the custom identification has finished, restoreCanvasPropertiesOverrides() should
      * be called to erase any overrides.
+     *
+     * \param searchRadiusMapUnits The overridden search radius in map units
      * \see restoreCanvasPropertiesOverrides()
      * \since QGIS 3.4
+     * \deprecated QGIS 3.42. Use setPropertiesOverrides() instead.
      */
-    void setCanvasPropertiesOverrides( double searchRadiusMapUnits );
+    Q_DECL_DEPRECATED void setCanvasPropertiesOverrides( double searchRadiusMapUnits ) SIP_DEPRECATED;
 
     /**
      * Clears canvas properties overrides previously set with setCanvasPropertiesOverrides()
      * \see setCanvasPropertiesOverrides()
      * \since QGIS 3.4
+     * \deprecated QGIS 3.42. Use restorePropertiesOverrides() instead.
      */
-    void restoreCanvasPropertiesOverrides();
+    Q_DECL_DEPRECATED void restoreCanvasPropertiesOverrides() SIP_DEPRECATED;
+
+    /**
+     * Overrides some map canvas properties inside the map tool for the upcoming identify requests.
+     *
+     * This is useful when the identification is triggered by some other piece of GUI like a 3D map view
+     * and some properties like search radius need to be adjusted so that identification returns correct
+     * results.
+     * When the custom identification has finished, restorePropertiesOverrides() should
+     * be called to erase any overrides.
+     *
+     * \param overrides The identify tool properties that will be overridden
+     * \since QGIS 3.42
+     */
+    void setPropertiesOverrides( IdentifyProperties overrides );
+
+    /**
+     * Clears canvas properties overrides previously set with setPropertiesOverrides()
+     * \see setPropertiesOverrides()
+     * \since QGIS 3.42
+     */
+    void restorePropertiesOverrides();
 
   private:
-
     bool identifyLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsMapLayer *layer, const QgsGeometry &geometry, const QgsRectangle &viewExtent, double mapUnitsPerPixel, QgsMapToolIdentify::LayerType layerType = AllLayers, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
     bool identifyRasterLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsRasterLayer *layer, const QgsGeometry &geometry, const QgsRectangle &viewExtent, double mapUnitsPerPixel, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
     bool identifyVectorLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsVectorLayer *layer, const QgsGeometry &geometry, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
-    int identifyVectorLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsVectorLayer *layer, const QgsFeatureList &features, QgsFeatureRenderer *renderer, const QMap< QString, QString >  &commonDerivedAttributes, const std::function< QMap< QString, QString > ( const QgsFeature & ) > &derivedAttributes, QgsRenderContext &context );
+    int identifyVectorLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsVectorLayer *layer, const QgsFeatureList &features, QgsFeatureRenderer *renderer, const QMap<QString, QString> &commonDerivedAttributes, const std::function<QMap<QString, QString>( const QgsFeature & )> &derivedAttributes, QgsRenderContext &context );
     bool identifyMeshLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsMeshLayer *layer, const QgsGeometry &geometry, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
     bool identifyVectorTileLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsVectorTileLayer *layer, const QgsGeometry &geometry, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
     bool identifyPointCloudLayer( QList<QgsMapToolIdentify::IdentifyResult> *results, QgsPointCloudLayer *layer, const QgsGeometry &geometry, const QgsIdentifyContext &identifyContext = QgsIdentifyContext() );
@@ -258,28 +303,24 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
     /**
      * Desired units for distance display.
      * \see displayAreaUnits()
-     * \since QGIS 2.14
      */
-    virtual QgsUnitTypes::DistanceUnit displayDistanceUnits() const;
+    virtual Qgis::DistanceUnit displayDistanceUnits() const;
 
     /**
      * Desired units for area display.
      * \see displayDistanceUnits()
-     * \since QGIS 2.14
      */
-    virtual QgsUnitTypes::AreaUnit displayAreaUnits() const;
+    virtual Qgis::AreaUnit displayAreaUnits() const;
 
     /**
      * Format a distance into a suitable string for display to the user
      * \see formatArea()
-     * \since QGIS 2.14
      */
     QString formatDistance( double distance ) const;
 
     /**
      * Format a distance into a suitable string for display to the user
      * \see formatDistance()
-     * \since QGIS 2.14
      */
     QString formatArea( double area ) const;
 
@@ -287,29 +328,28 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
      * Format a distance into a suitable string for display to the user
      * \see formatArea()
      */
-    QString formatDistance( double distance, QgsUnitTypes::DistanceUnit unit ) const;
+    QString formatDistance( double distance, Qgis::DistanceUnit unit ) const;
 
     /**
      * Format a distance into a suitable string for display to the user
      * \see formatDistance()
      */
-    QString formatArea( double area, QgsUnitTypes::AreaUnit unit ) const;
+    QString formatArea( double area, Qgis::AreaUnit unit ) const;
 
-    QMap< QString, QString > featureDerivedAttributes( const QgsFeature &feature, QgsMapLayer *layer, const QgsPointXY &layerPoint = QgsPointXY() );
+    QMap<QString, QString> featureDerivedAttributes( const QgsFeature &feature, QgsMapLayer *layer, const QgsPointXY &layerPoint = QgsPointXY() );
 
     /**
      * Adds details of the closest vertex to derived attributes
      */
-    void closestVertexAttributes( const QgsAbstractGeometry &geometry, QgsVertexId vId, QgsMapLayer *layer, QMap< QString, QString > &derivedAttributes );
+    void closestVertexAttributes( const QgsCoordinateTransform layerToMapTransform, const QgsCoordinateReferenceSystem &layerVertCrs, const QgsCoordinateReferenceSystem &mapVertCrs, const QgsAbstractGeometry &geometry, QgsVertexId vId, bool showTransformedZ, QMap<QString, QString> &derivedAttributes );
 
     /**
      * Adds details of the closest point to derived attributes
     */
-    void closestPointAttributes( const QgsAbstractGeometry &geometry, const QgsPointXY &layerPoint, QMap< QString, QString > &derivedAttributes );
+    void closestPointAttributes( const QgsCoordinateTransform layerToMapTransform, const QgsCoordinateReferenceSystem &layerVertCrs, const QgsCoordinateReferenceSystem &mapVertCrs, const QgsAbstractGeometry &geometry, const QgsPointXY &layerPoint, bool showTransformedZ, QMap<QString, QString> &derivedAttributes );
 
-    QString formatCoordinate( const QgsPointXY &canvasPoint ) const;
-    QString formatXCoordinate( const QgsPointXY &canvasPoint ) const;
-    QString formatYCoordinate( const QgsPointXY &canvasPoint ) const;
+    static void formatCoordinate( const QgsPointXY &canvasPoint, QString &x, QString &y, const QgsCoordinateReferenceSystem &mapCrs, int coordinatePrecision );
+    void formatCoordinate( const QgsPointXY &canvasPoint, QString &x, QString &y ) const;
 
     // Last geometry (point or polygon) in map CRS
     QgsGeometry mLastGeometry;
@@ -320,7 +360,7 @@ class GUI_EXPORT QgsMapToolIdentify : public QgsMapTool
 
     int mCoordinatePrecision;
 
-    double mOverrideCanvasSearchRadius = -1;
+    IdentifyProperties mPropertiesOverrides;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMapToolIdentify::LayerType )
