@@ -29,8 +29,9 @@ class QgsSettingsEntryString;
 /**
  * \ingroup core
  * \class QgsSettingsTreeNode
- * \brief QgsSettingsTreeNode is a tree node for the settings tree
+ * \brief A tree node for the settings tree
  * to help organizing and introspecting the tree.
+ *
  * It is either a root node, a normal node or
  * a named list (to store a group of settings under a dynamic named key).
  * to automatically register a settings entry on its creation when a parent is provided.
@@ -147,6 +148,8 @@ class CORE_EXPORT QgsSettingsTreeNode
   private:
 
     /**
+     * Default constructor for QgsSettingsTreeNode.
+     *
      * \note This is not available in Python bindings. Use method createNode on an existing tree node.
      * \see QgsSettingsTree.createPluginTreeNode
      */
@@ -176,9 +179,10 @@ class CORE_EXPORT QgsSettingsTreeNode
 /**
  * \ingroup core
  * \class QgsSettingsTreeNamedListNode
- * \brief QgsSettingsTreeNamedListNode is a named list tree node for the settings tree
+ * \brief A named list tree node for the settings tree
  * to help organizing and introspecting the tree.
- * the named list node is used to store a group of settings under a dynamically named key.
+ *
+ * The named list node is used to store a group of settings under a dynamically named key.
  *
  * \see QgsSettingsTree
  * \see QgsSettingsTreeNode
@@ -239,7 +243,7 @@ class CORE_EXPORT QgsSettingsTreeNamedListNode : public QgsSettingsTreeNode
     void deleteAllItems( const QStringList &parentsNamedItems = QStringList() ) SIP_THROW( QgsSettingsException );
 
     //! Returns the setting used to store the selected item
-    const QgsSettingsEntryString *selectedItemSetting() const {return mSelectedItemSetting;}
+    const QgsSettingsEntryString *selectedItemSetting() const {return mSelectedItemSetting.get();}
 
   protected:
     //! Init the nodes with the specific \a options
@@ -249,18 +253,20 @@ class CORE_EXPORT QgsSettingsTreeNamedListNode : public QgsSettingsTreeNode
     friend class QgsSettingsTreeNode;
 
     /**
+     * Constructor for QgsSettingsTreeNamedListNode.
+     *
      * \note This is not available in Python bindings. Use method createNamedListNode on an existing tree node.
      * \see QgsSettingsTree.createPluginTreeNode
      */
-    QgsSettingsTreeNamedListNode() = default SIP_FORCE;
+    QgsSettingsTreeNamedListNode() SIP_FORCE;
 
-    QgsSettingsTreeNamedListNode( const QgsSettingsTreeNamedListNode &other ) = default SIP_FORCE;
+    QgsSettingsTreeNamedListNode( const QgsSettingsTreeNamedListNode &other ) = delete;
 
     //! Returns the key with named items placeholders filled with args
     QString completeKeyWithNamedItems( const QString &key, const QStringList &namedItems ) const;
 
     Qgis::SettingsTreeNodeOptions mOptions;
-    const QgsSettingsEntryString *mSelectedItemSetting = nullptr;
+    std::unique_ptr<const QgsSettingsEntryString> mSelectedItemSetting;
     QString mItemsCompleteKey;
 };
 
