@@ -23,6 +23,7 @@
 #include "qgis.h"
 
 #include <QPicture>
+#include <QPainterPath>
 
 class QgsTextDocument;
 class QgsTextDocumentMetrics;
@@ -36,7 +37,6 @@ class QFontMetricsF;
   * \ingroup core
   * \brief Handles rendering text using rich formatting options, including drop shadows, buffers
   * and background shapes.
-  * \since QGIS 3.0
  */
 class CORE_EXPORT QgsTextRenderer
 {
@@ -144,6 +144,35 @@ class CORE_EXPORT QgsTextRenderer
                           QgsRenderContext &context, const QgsTextFormat &format,
                           bool drawAsOutlines = true );
 
+    /**
+     * Draws a text document at a point origin using the specified settings.
+     *
+     * Calling this method is more efficient than calling drawText() if the text document and metrics have already
+     * been calculated.
+     *
+     * \warning Unlike drawText(), this method does not automatically update data defined properties in the text \a format. This
+     * is the caller's responsibility to do, and must be done prior to generating the text \a document and \a metrics.
+     *
+     * \param point origin of text, in painter units
+     * \param format base text format
+     * \param document text document to draw
+     * \param metrics precalculated text metrics
+     * \param context destination render context
+     * \param alignment horizontal alignment
+     * \param rotation text rotation
+     * \param mode optional layout mode (since QGIS 3.42)
+     *
+     * \since QGIS 3.40
+     */
+    static void drawDocument( QPointF point,
+                              const QgsTextFormat &format,
+                              const QgsTextDocument &document,
+                              const QgsTextDocumentMetrics &metrics,
+                              QgsRenderContext &context,
+                              Qgis::TextHorizontalAlignment alignment,
+                              double rotation,
+                              Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Point );
+
     /**
      * Draws text along a line using the specified settings.
      *
@@ -153,12 +182,14 @@ class CORE_EXPORT QgsTextRenderer
      * \param format text format
      * \param offsetAlongLine offset along the line (in painter units) to start text at
      * \param offsetFromLine offset from the line (in painter units). Negative values will shift the text to the left of the line, positive values will shift the text to the right.
+     * \param flags curved text behavior flags (since QGIS 4.0)
      *
      * \since QGIS 3.32
      */
     static void drawTextOnLine( const QPolygonF &line, const QString &text,
                                 QgsRenderContext &context, const QgsTextFormat &format,
-                                double offsetAlongLine = 0, double offsetFromLine = 0 );
+                                double offsetAlongLine = 0, double offsetFromLine = 0,
+                                Qgis::CurvedTextFlags flags = Qgis::CurvedTextFlag::UseBaselinePlacement | Qgis::CurvedTextFlag::TruncateStringWhenLineIsTooShort );
 
     /**
      * Draws a text document along a line using the specified settings.
@@ -169,6 +200,7 @@ class CORE_EXPORT QgsTextRenderer
      * \param context render context
      * \param offsetAlongLine offset along the line (in painter units) to start text at
      * \param offsetFromLine offset from the line (in painter units). Negative values will shift the text to the left of the line, positive values will shift the text to the right.
+     * \param flags curved text behavior flags (since QGIS 4.0)
      *
      * \since QGIS 3.32
      */
@@ -177,7 +209,8 @@ class CORE_EXPORT QgsTextRenderer
                                     const QgsTextDocument &document,
                                     QgsRenderContext &context,
                                     double offsetAlongLine = 0,
-                                    double offsetFromLine = 0 );
+                                    double offsetFromLine = 0,
+                                    Qgis::CurvedTextFlags flags = Qgis::CurvedTextFlag::UseBaselinePlacement | Qgis::CurvedTextFlag::TruncateStringWhenLineIsTooShort );
 
     /**
      * Draws a single component of rendered text using the specified settings.
@@ -195,7 +228,7 @@ class CORE_EXPORT QgsTextRenderer
      * rendering and may result in side effects like misaligned text buffers. This setting is deprecated and has no effect
      * as of QGIS 3.4.3 and the text format should be set using QgsRenderContext::setTextRenderFormat() instead.
      *
-     * \deprecated Private API only, will be removed in 4.0
+     * \deprecated QGIS 3.40. Private API only, will be removed in 4.0.
      */
     Q_DECL_DEPRECATED static void drawPart( const QRectF &rect, double rotation, Qgis::TextHorizontalAlignment alignment, const QStringList &textLines,
                                             QgsRenderContext &context, const QgsTextFormat &format,
@@ -217,7 +250,7 @@ class CORE_EXPORT QgsTextRenderer
      * rendering and may result in side effects like misaligned text buffers. This setting is deprecated and has no effect
      * as of QGIS 3.4.3 and the text format should be set using QgsRenderContext::setTextRenderFormat() instead.
      *
-     * \deprecated Private API only, will be removed in 4.0
+     * \deprecated QGIS 3.40. Private API only, will be removed in 4.0.
      */
     Q_DECL_DEPRECATED static void drawPart( QPointF origin, double rotation, Qgis::TextHorizontalAlignment alignment, const QStringList &textLines,
                                             QgsRenderContext &context, const QgsTextFormat &format,
@@ -229,7 +262,7 @@ class CORE_EXPORT QgsTextRenderer
      * all scaling required by the render context.
      *
      * The optional \a scaleFactor argument can specify a font size scaling factor. It is recommended to set this to
-     * QgsTextRenderer::FONT_WORKAROUND_SCALE and then manually scale painter devices or calculations
+     * QgsTextRenderer::calculateScaleFactorForFormat() and then manually scale painter devices or calculations
      * based on the resultant font metrics. Failure to do so will result in poor quality text rendering
      * at small font sizes.
      *
@@ -295,10 +328,22 @@ class CORE_EXPORT QgsTextRenderer
      * Using this scale factor and manually adjusting any font metric based calculations results in more stable
      * font metrics and sizes for small font sizes.
      *
+     * \warning Deprecated, use calculateScaleFactorForFormat() instead.
+     *
      * \since QGIS 3.16
      */
     static constexpr double FONT_WORKAROUND_SCALE = 10;
 
+    /**
+     * Returns the scale factor used for upscaling font sizes and downscaling destination painter devices.
+     *
+     * Using this scale factor and manually adjusting any font metric based calculations results in more stable
+     * font metrics and sizes for small font sizes.
+     *
+     * \since QGIS 3.40
+     */
+    static double calculateScaleFactorForFormat( const QgsRenderContext &context, const QgsTextFormat &format );
+
     // to match QTextEngine handling of superscript/subscript font sizes
 
     /**
@@ -358,7 +403,7 @@ class CORE_EXPORT QgsTextRenderer
     static double textHeight( const QgsRenderContext &context, const QgsTextFormat &format, const QgsTextDocument &document, Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Point );
 
     /**
-     * Draws a single component of rendered text using the specified settings.
+     * Draws components of rendered text using the specified settings.
      * \param rect destination rectangle for text
      * \param rotation text rotation
      * \param alignment horizontal alignment
@@ -367,19 +412,19 @@ class CORE_EXPORT QgsTextRenderer
      * \param metrics document metrics
      * \param context render context
      * \param format text format
-     * \param part component of text to draw. Note that Shadow parts cannot be drawn
+     * \param parts components of text to draw. Note that Shadow parts cannot be drawn
      * individually and instead are drawn with their associated part (e.g., drawn together
      * with the text or background parts)
      * \param mode layout mode
      * \note Not available in Python bindings
      * \since QGIS 3.14
      */
-    static void drawPart( const QRectF &rect, double rotation, Qgis::TextHorizontalAlignment alignment, Qgis::TextVerticalAlignment vAlignment, const QgsTextDocument &document, const QgsTextDocumentMetrics &metrics,
-                          QgsRenderContext &context, const QgsTextFormat &format,
-                          Qgis::TextComponent part, Qgis::TextLayoutMode mode );
+    static void drawParts( const QRectF &rect, double rotation, Qgis::TextHorizontalAlignment alignment, Qgis::TextVerticalAlignment vAlignment, const QgsTextDocument &document, const QgsTextDocumentMetrics &metrics,
+                           QgsRenderContext &context, const QgsTextFormat &format,
+                           Qgis::TextComponents parts, Qgis::TextLayoutMode mode );
 
     /**
-     * Draws a single component of rendered text using the specified settings.
+     * Draws components of rendered text using the specified settings.
      * \param origin origin for start of text. Y coordinate will be used as baseline.
      * \param rotation text rotation
      * \param alignment horizontal alignment
@@ -387,18 +432,18 @@ class CORE_EXPORT QgsTextRenderer
      * \param metrics precalculated document metrics
      * \param context render context
      * \param format text format
-     * \param part component of text to draw. Note that Shadow parts cannot be drawn
+     * \param parts components of text to draw. Note that Shadow parts cannot be drawn
      * individually and instead are drawn with their associated part (e.g., drawn together
      * with the text or background parts)
      * \param mode layout mode
      * \note Not available in Python bindings
      * \since QGIS 3.14
      */
-    static void drawPart( QPointF origin, double rotation, Qgis::TextHorizontalAlignment alignment, const QgsTextDocument &document,
-                          const QgsTextDocumentMetrics &metrics,
-                          QgsRenderContext &context, const QgsTextFormat &format,
-                          Qgis::TextComponent part,
-                          Qgis::TextLayoutMode mode );
+    static void drawParts( QPointF origin, double rotation, Qgis::TextHorizontalAlignment alignment, const QgsTextDocument &document,
+                           const QgsTextDocumentMetrics &metrics,
+                           QgsRenderContext &context, const QgsTextFormat &format,
+                           Qgis::TextComponents parts,
+                           Qgis::TextLayoutMode mode );
 
     static double drawBuffer( QgsRenderContext &context,
                               const Component &component,
@@ -407,7 +452,7 @@ class CORE_EXPORT QgsTextRenderer
                               Qgis::TextLayoutMode mode );
 
     static void drawBackground( QgsRenderContext &context,
-                                Component component,
+                                const Component &component,
                                 const QgsTextFormat &format,
                                 const QgsTextDocumentMetrics &metrics,
                                 Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Rectangle );
@@ -426,7 +471,7 @@ class CORE_EXPORT QgsTextRenderer
                           const Component &component,
                           const QgsTextFormat &format );
 
-    static void drawTextInternal( Qgis::TextComponent drawType,
+    static void drawTextInternal( Qgis::TextComponents components,
                                   QgsRenderContext &context,
                                   const QgsTextFormat &format,
                                   const Component &component,
@@ -443,7 +488,7 @@ class CORE_EXPORT QgsTextRenderer
 
     static void drawTextInternalHorizontal( QgsRenderContext &context,
                                             const QgsTextFormat &format,
-                                            Qgis::TextComponent drawType,
+                                            Qgis::TextComponents components,
                                             Qgis::TextLayoutMode mode,
                                             const Component &component,
                                             const QgsTextDocument &document,
@@ -455,7 +500,7 @@ class CORE_EXPORT QgsTextRenderer
 
     static void drawTextInternalVertical( QgsRenderContext &context,
                                           const QgsTextFormat &format,
-                                          Qgis::TextComponent drawType,
+                                          Qgis::TextComponents components,
                                           Qgis::TextLayoutMode mode,
                                           const Component &component,
                                           const QgsTextDocument &document,
@@ -465,13 +510,67 @@ class CORE_EXPORT QgsTextRenderer
                                           Qgis::TextVerticalAlignment vAlignment,
                                           double rotation );
 
-    static double calculateScaleFactorForFormat( const QgsRenderContext &context, const QgsTextFormat &format );
+    struct DeferredRenderFragment
+    {
+      // mandatory
+      QColor color;
+      QPointF point;
+      // optional
+      QPainterPath path;
+      // optional
+      QFont font;
+      QString text;
+    };
+
+    struct BlockMetrics
+    {
+      double xOffset = 0;
+      double backgroundXOffset = 0;
+      double width = 0;
+      double backgroundWidth = 0;
+      double extraWordSpace = 0;
+      double extraLetterSpace = 0;
+    };
+
+    static QVector< QgsTextRenderer::BlockMetrics > calculateBlockMetrics( const QgsTextDocument &document, const QgsTextDocumentMetrics &metrics, Qgis::TextLayoutMode mode, double targetWidth, const Qgis::TextHorizontalAlignment hAlignment );
+
+    struct DeferredRenderBlock
+    {
+      QPointF origin;
+      Component component;
+      QVector< DeferredRenderFragment > fragments;
+    };
+
+    static QBrush createBrushForPath( QgsRenderContext &context, const QString &path );
+
+    static void renderBlockHorizontal( const QgsTextBlock &block, int blockIndex,
+                                       const QgsTextDocumentMetrics &metrics, QgsRenderContext &context,
+                                       const QgsTextFormat &format,
+                                       QPainter *painter, bool forceRenderAsPaths,
+                                       double fontScale, double extraWordSpace, double extraLetterSpace,
+                                       Qgis::TextLayoutMode mode,
+                                       DeferredRenderBlock *deferredRenderBlock );
+    static void renderDocumentBackgrounds( QgsRenderContext &context, const QgsTextDocument &document, const QgsTextDocumentMetrics &metrics, const Component &component, const QVector< QgsTextRenderer::BlockMetrics > &blockMetrics, Qgis::TextLayoutMode mode, double verticalAlignOffset, double rotation );
+    static void renderDeferredBlocks( QgsRenderContext &context, const QgsTextFormat &format, Qgis::TextComponents components, const std::vector<DeferredRenderBlock> &deferredBlocks, bool usePathsForText, double fontScale, const Component &component, double rotation );
+    static void renderDeferredBuffer( QgsRenderContext &context, const QgsTextFormat &format, Qgis::TextComponents components, const std::vector<DeferredRenderBlock> &deferredBlocks, double fontScale, const Component &component, double rotation );
+    static void renderDeferredShadowForText( QgsRenderContext &context, const QgsTextFormat &format, const std::vector<DeferredRenderBlock> &deferredBlocks, double fontScale, const Component &component, double rotation );
+    static void renderDeferredText( QgsRenderContext &context, const std::vector<DeferredRenderBlock> &deferredBlocks, bool usePathsForText, double fontScale, const Component &component, double rotation );
 
     friend class QgsVectorLayerLabelProvider;
     friend class QgsLabelPreview;
 
     static QgsTextFormat updateShadowPosition( const QgsTextFormat &format );
 
+    /**
+     * Returns TRUE if paths should be used to render a document
+     */
+    static bool usePathsToRender( const QgsRenderContext &context, const QgsTextFormat &format, const QgsTextDocument &document );
+
+    /**
+     * Returns TRUE if a picture should be used to render a document
+     */
+    static bool usePictureToRender( const QgsRenderContext &context, const QgsTextFormat &format, const QgsTextDocument &document );
+
 };
 
 #endif // QGSTEXTRENDERER_H
