@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 /***************************************************************************
 Name                 : DB Manager
@@ -23,18 +21,30 @@ email                : brush.tyler@gmail.com
 # this will disable the dbplugin if the connector raise an ImportError
 from .connector import PostGisDBConnector
 
-from qgis.PyQt.QtCore import QSettings, Qt, QRegExp
+from qgis.PyQt.QtCore import Qt, QRegularExpression, QCoreApplication
 from qgis.PyQt.QtGui import QIcon
 from qgis.PyQt.QtWidgets import QAction, QApplication, QMessageBox
+from qgis.core import Qgis, QgsApplication, QgsSettings
 from qgis.gui import QgsMessageBar
 
-from ..plugin import ConnectionError, InvalidDataException, DBPlugin, Database, Schema, Table, VectorTable, RasterTable, \
-    TableField, TableConstraint, TableIndex, TableTrigger, TableRule
+from ..plugin import (
+    ConnectionError,
+    InvalidDataException,
+    DBPlugin,
+    Database,
+    Schema,
+    Table,
+    VectorTable,
+    RasterTable,
+    TableField,
+    TableConstraint,
+    TableIndex,
+    TableTrigger,
+    TableRule,
+)
 
 import re
 
-from . import resources_rc  # NOQA
-
 
 def classFactory():
     return PostGisDBPlugin
@@ -44,51 +54,73 @@ class PostGisDBPlugin(DBPlugin):
 
     @classmethod
     def icon(self):
-        return QIcon(":/db_manager/postgis/icon")
+        return QgsApplication.getThemeIcon("/mIconPostgis.svg")
 
     @classmethod
     def typeName(self):
-        return 'postgis'
+        return "postgis"
 
     @classmethod
     def typeNameString(self):
-        return 'PostGIS'
+        return QCoreApplication.translate("db_manager", "PostgreSQL")
 
     @classmethod
     def providerName(self):
-        return 'postgres'
+        return "postgres"
 
     @classmethod
     def connectionSettingsKey(self):
-        return '/PostgreSQL/connections'
+        return "/PostgreSQL/connections"
 
     def databasesFactory(self, connection, uri):
         return PGDatabase(connection, uri)
 
     def connect(self, parent=None):
         conn_name = self.connectionName()
-        settings = QSettings()
-        settings.beginGroup(u"/%s/%s" % (self.connectionSettingsKey(), conn_name))
+        settings = QgsSettings()
+        settings.beginGroup(f"/{self.connectionSettingsKey()}/{conn_name}")
 
         if not settings.contains("database"):  # non-existent entry?
-            raise InvalidDataException(self.tr('There is no defined database connection "%s".') % conn_name)
+            raise InvalidDataException(
+                self.tr('There is no defined database connection "{0}".').format(
+                    conn_name
+                )
+            )
 
-        from qgis.core import QgsDataSourceURI
+        from qgis.core import QgsDataSourceUri
 
-        uri = QgsDataSourceURI()
+        uri = QgsDataSourceUri()
 
-        settingsList = ["service", "host", "port", "database", "username", "password", "authcfg"]
-        service, host, port, database, username, password, authcfg = [settings.value(x, "", type=str) for x in settingsList]
+        settingsList = [
+            "service",
+            "host",
+            "port",
+            "database",
+            "username",
+            "password",
+            "authcfg",
+        ]
+        service, host, port, database, username, password, authcfg = (
+            settings.value(x, "", type=str) for x in settingsList
+        )
 
         useEstimatedMetadata = settings.value("estimatedMetadata", False, type=bool)
-        sslmode = settings.value("sslmode", QgsDataSourceURI.SSLprefer, type=int)
+        try:
+            sslmode = settings.enumValue("sslmode", QgsDataSourceUri.SslMode.SslPrefer)
+        except TypeError:
+            sslmode = QgsDataSourceUri.SslMode.SslPrefer
 
         settings.endGroup()
 
+        if hasattr(authcfg, "isNull") and authcfg.isNull():
+            authcfg = ""
+
         if service:
             uri.setConnection(service, database, username, password, sslmode, authcfg)
         else:
-            uri.setConnection(host, port, database, username, password, sslmode, authcfg)
+            uri.setConnection(
+                host, port, database, username, password, sslmode, authcfg
+            )
 
         uri.setUseEstimatedMetadata(useEstimatedMetadata)
 
@@ -104,13 +136,14 @@ class PGDatabase(Database):
         Database.__init__(self, connection, uri)
 
     def connectorsFactory(self, uri):
-        return PostGisDBConnector(uri)
+        return PostGisDBConnector(uri, self.connection())
 
     def dataTablesFactory(self, row, db, schema=None):
         return PGTable(row, db, schema)
 
     def info(self):
         from .info_model import PGDatabaseInfo
+
         return PGDatabaseInfo(self)
 
     def vectorTablesFactory(self, row, db, schema=None):
@@ -127,6 +160,11 @@ class PGDatabase(Database):
 
         return PGSqlResultModel(self, sql, parent)
 
+    def sqlResultModelAsync(self, sql, parent):
+        from .data_model import PGSqlResultModelAsync
+
+        return PGSqlResultModelAsync(self, sql, parent)
+
     def registerDatabaseActions(self, mainWindow):
         Database.registerDatabaseActions(self, mainWindow)
 
@@ -136,38 +174,54 @@ class PGDatabase(Database):
         mainWindow.registerAction(separator, self.tr("&Table"))
 
         action = QAction(self.tr("Run &Vacuum Analyze"), self)
-        mainWindow.registerAction(action, self.tr("&Table"), self.runVacuumAnalyzeActionSlot)
+        mainWindow.registerAction(
+            action, self.tr("&Table"), self.runVacuumAnalyzeActionSlot
+        )
 
         action = QAction(self.tr("Run &Refresh Materialized View"), self)
-        mainWindow.registerAction(action, self.tr("&Table"), self.runRefreshMaterializedViewSlot)
+        mainWindow.registerAction(
+            action, self.tr("&Table"), self.runRefreshMaterializedViewSlot
+        )
 
     def runVacuumAnalyzeActionSlot(self, item, action, parent):
         QApplication.restoreOverrideCursor()
         try:
             if not isinstance(item, Table) or item.isView:
-                parent.infoBar.pushMessage(self.tr("Select a table for vacuum analyze."), QgsMessageBar.INFO,
-                                           parent.iface.messageTimeout())
+                parent.infoBar.pushMessage(
+                    self.tr("Select a table for vacuum analyze."),
+                    Qgis.MessageLevel.Info,
+                    parent.iface.messageTimeout(),
+                )
                 return
         finally:
-            QApplication.setOverrideCursor(Qt.WaitCursor)
+            QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
 
         item.runVacuumAnalyze()
 
     def runRefreshMaterializedViewSlot(self, item, action, parent):
         QApplication.restoreOverrideCursor()
         try:
-            if not isinstance(item, PGTable) or item._relationType != 'm':
-                parent.infoBar.pushMessage(self.tr("Select a materialized view for refresh."), QgsMessageBar.INFO,
-                                           parent.iface.messageTimeout())
+            if not isinstance(item, PGTable) or item._relationType != "m":
+                parent.infoBar.pushMessage(
+                    self.tr("Select a materialized view for refresh."),
+                    Qgis.MessageLevel.Info,
+                    parent.iface.messageTimeout(),
+                )
                 return
         finally:
-            QApplication.setOverrideCursor(Qt.WaitCursor)
+            QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
 
         item.runRefreshMaterializedView()
 
     def hasLowercaseFieldNamesOption(self):
         return True
 
+    def supportsComment(self):
+        return True
+
+    def executeSql(self, sql):
+        return self.connector._executeSql(sql)
+
 
 class PGSchema(Schema):
 
@@ -180,8 +234,16 @@ class PGTable(Table):
 
     def __init__(self, row, db, schema=None):
         Table.__init__(self, db, schema)
-        self.name, schema_name, self._relationType, self.owner, self.estimatedRowCount, self.pages, self.comment = row
-        self.isView = self._relationType in set(['v', 'm'])
+        (
+            self.name,
+            schema_name,
+            self._relationType,
+            self.owner,
+            self.estimatedRowCount,
+            self.pages,
+            self.comment,
+        ) = row
+        self.isView = self._relationType in {"v", "m"}
         self.estimatedRowCount = int(self.estimatedRowCount)
 
     def runVacuumAnalyze(self):
@@ -192,12 +254,14 @@ class PGTable(Table):
 
     def runRefreshMaterializedView(self):
         self.aboutToChange.emit()
-        self.database().connector.runRefreshMaterializedView((self.schemaName(), self.name))
+        self.database().connector.runRefreshMaterializedView(
+            (self.schemaName(), self.name)
+        )
         # TODO: change only this item, not re-create all the tables in the schema/database
         self.schema().refresh() if self.schema() else self.database().refresh()
 
     def runAction(self, action):
-        action = unicode(action)
+        action = str(action)
 
         if action.startswith("vacuumanalyze/"):
             if action == "vacuumanalyze/run":
@@ -205,24 +269,33 @@ class PGTable(Table):
                 return True
 
         elif action.startswith("rule/"):
-            parts = action.split('/')
+            parts = action.split("/")
             rule_name = parts[1]
             rule_action = parts[2]
 
-            msg = u"Do you want to %s rule %s?" % (rule_action, rule_name)
+            msg = self.tr("Do you want to {0} rule {1}?").format(rule_action, rule_name)
 
             QApplication.restoreOverrideCursor()
 
             try:
-                if QMessageBox.question(None, self.tr("Table rule"), msg,
-                                        QMessageBox.Yes | QMessageBox.No) == QMessageBox.No:
+                if (
+                    QMessageBox.question(
+                        None,
+                        self.tr("Table rule"),
+                        msg,
+                        QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
+                    )
+                    == QMessageBox.StandardButton.No
+                ):
                     return False
             finally:
-                QApplication.setOverrideCursor(Qt.WaitCursor)
+                QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
 
             if rule_action == "delete":
                 self.aboutToChange.emit()
-                self.database().connector.deleteTableRule(rule_name, (self.schemaName(), self.name))
+                self.database().connector.deleteTableRule(
+                    rule_name, (self.schemaName(), self.name)
+                )
                 self.refreshRules()
                 return True
 
@@ -253,6 +326,9 @@ class PGTable(Table):
 
         return PGTableInfo(self)
 
+    def crs(self):
+        return self.database().connector.getCrs(self.srid)
+
     def tableDataModel(self, parent):
         from .data_model import PGTableDataModel
 
@@ -261,7 +337,9 @@ class PGTable(Table):
     def delete(self):
         self.aboutToChange.emit()
         if self.isView:
-            ret = self.database().connector.deleteView((self.schemaName(), self.name), self._relationType == 'm')
+            ret = self.database().connector.deleteView(
+                (self.schemaName(), self.name), self._relationType == "m"
+            )
         else:
             ret = self.database().connector.deleteTable((self.schemaName(), self.name))
         if not ret:
@@ -286,69 +364,124 @@ class PGVectorTable(PGTable, VectorTable):
             return True
         return VectorTable.runAction(self, action)
 
+    def geometryType(self):
+        """Returns the proper WKT type.
+        PostGIS records type like this:
+        | WKT Type     | geomType    | geomDim |
+        |--------------|-------------|---------|
+        | LineString   | LineString  | 2       |
+        | LineStringZ  | LineString  | 3       |
+        | LineStringM  | LineStringM | 3       |
+        | LineStringZM | LineString  | 4       |
+        """
+        geometryType = self.geomType
+        if self.geomDim == 3 and self.geomType[-1] != "M":
+            geometryType += "Z"
+        elif self.geomDim == 4:
+            geometryType += "ZM"
+
+        return geometryType
+
 
 class PGRasterTable(PGTable, RasterTable):
 
     def __init__(self, row, db, schema=None):
         PGTable.__init__(self, row[:-6], db, schema)
         RasterTable.__init__(self, db, schema)
-        self.geomColumn, self.pixelType, self.pixelSizeX, self.pixelSizeY, self.isExternal, self.srid = row[-6:]
-        self.geomType = 'RASTER'
+        (
+            self.geomColumn,
+            self.pixelType,
+            self.pixelSizeX,
+            self.pixelSizeY,
+            self.isExternal,
+            self.srid,
+        ) = row[-6:]
+        self.geomType = "RASTER"
 
     def info(self):
         from .info_model import PGRasterTableInfo
 
         return PGRasterTableInfo(self)
 
-    def gdalUri(self, uri=None):
+    def uri(self, uri=None):
+        """Returns the datasource URI for postgresraster provider"""
+
         if not uri:
             uri = self.database().uri()
-        schema = (u'schema=%s' % self.schemaName()) if self.schemaName() else ''
-        dbname = (u'dbname=%s' % uri.database()) if uri.database() else ''
-        host = (u'host=%s' % uri.host()) if uri.host() else ''
-        user = (u'user=%s' % uri.username()) if uri.username() else ''
-        passw = (u'password=%s' % uri.password()) if uri.password() else ''
-        port = (u'port=%s' % uri.port()) if uri.port() else ''
+        service = ("service='%s'" % uri.service()) if uri.service() else ""
+        dbname = ("dbname='%s'" % uri.database()) if uri.database() else ""
+        host = ("host=%s" % uri.host()) if uri.host() else ""
+        user = ("user=%s" % uri.username()) if uri.username() else ""
+        passw = ("password=%s" % uri.password()) if uri.password() else ""
+        port = ("port=%s" % uri.port()) if uri.port() else ""
+
+        schema = self.schemaName() if self.schemaName() else "public"
+        table = f'"{schema}"."{self.name}"'
+
+        if not dbname:
+            # postgresraster provider *requires* a dbname
+            connector = self.database().connector
+            r = connector._execute(None, "SELECT current_database()")
+            dbname = "dbname='%s'" % connector._fetchone(r)[0]
+            connector._close_cursor(r)
 
         # Find first raster field
-        col = ''
+        col = ""
         for fld in self.fields():
             if fld.dataType == "raster":
-                col = u'column=%s' % fld.name
+                col = "column='%s'" % fld.name
                 break
 
-        gdalUri = u'PG: %s %s %s %s %s mode=2 %s %s table=%s' % \
-                  (dbname, host, user, passw, port, schema, col, self.name)
+        uri = "{} {} {} {} {} {} {} table={}".format(
+            service,
+            dbname,
+            host,
+            user,
+            passw,
+            port,
+            col,
+            table,
+        )
 
-        return gdalUri
-
-    def mimeUri(self):
-        # QGIS has no provider for PGRasters, let's use GDAL
-        uri = u"raster:gdal:%s:%s" % (self.name, re.sub(":", "\:", self.gdalUri()))
         return uri
 
-    def toMapLayer(self):
-        from qgis.core import QgsRasterLayer, QgsContrastEnhancement, QgsDataSourceURI, QgsCredentials
+    def mimeUri(self):
+        uri = "raster:postgresraster:{}:{}".format(
+            self.name, re.sub(":", r"\:", self.uri())
+        )
+        return uri
 
-        rl = QgsRasterLayer(self.gdalUri(), self.name)
+    def toMapLayer(self, geometryType=None, crs=None):
+        from qgis.core import (
+            QgsRasterLayer,
+            QgsContrastEnhancement,
+            QgsDataSourceUri,
+            QgsCredentials,
+        )
+
+        rl = QgsRasterLayer(self.uri(), self.name, "postgresraster")
         if not rl.isValid():
             err = rl.error().summary()
-            uri = QgsDataSourceURI(self.database().uri())
+            uri = QgsDataSourceUri(self.database().uri())
             conninfo = uri.connectionInfo(False)
             username = uri.username()
             password = uri.password()
 
             for i in range(3):
-                (ok, username, password) = QgsCredentials.instance().get(conninfo, username, password, err)
+                (ok, username, password) = QgsCredentials.instance().get(
+                    conninfo, username, password, err
+                )
                 if ok:
                     uri.setUsername(username)
                     uri.setPassword(password)
-                    rl = QgsRasterLayer(self.gdalUri(uri), self.name)
+                    rl = QgsRasterLayer(self.uri(uri), self.name)
                     if rl.isValid():
                         break
 
         if rl.isValid():
-            rl.setContrastEnhancement(QgsContrastEnhancement.StretchToMinimumMaximum)
+            rl.setContrastEnhancement(
+                QgsContrastEnhancement.ContrastEnhancementAlgorithm.StretchToMinimumMaximum
+            )
         return rl
 
 
@@ -356,15 +489,25 @@ class PGTableField(TableField):
 
     def __init__(self, row, table):
         TableField.__init__(self, table)
-        self.num, self.name, self.dataType, self.charMaxLen, self.modifier, self.notNull, self.hasDefault, self.default, typeStr = row
+        (
+            self.num,
+            self.name,
+            self.dataType,
+            self.charMaxLen,
+            self.modifier,
+            self.notNull,
+            self.hasDefault,
+            self.default,
+            typeStr,
+        ) = row
         self.primaryKey = False
 
         # get modifier (e.g. "precision,scale") from formatted type string
         trimmedTypeStr = typeStr.strip()
-        regex = QRegExp("\((.+)\)$")
-        startpos = regex.indexIn(trimmedTypeStr)
-        if startpos >= 0:
-            self.modifier = regex.cap(1).strip()
+        regex = QRegularExpression(r"\((.+)\)$")
+        match = regex.match(trimmedTypeStr)
+        if match.hasMatch():
+            self.modifier = match.captured(1).strip()
         else:
             self.modifier = None
 
@@ -374,13 +517,37 @@ class PGTableField(TableField):
                 self.primaryKey = True
                 break
 
+    def getComment(self):
+        """Returns the comment for a field"""
+        tab = self.table()
+        # SQL Query checking if a comment exists for the field
+        sql_cpt = "Select count(*) from pg_description pd, pg_class pc, pg_attribute pa where relname = '{}' and attname = '{}' and pa.attrelid = pc.oid and pd.objoid = pc.oid and pd.objsubid = pa.attnum".format(
+            tab.name, self.name
+        )
+        # SQL Query that return the comment of the field
+        sql = "Select pd.description from pg_description pd, pg_class pc, pg_attribute pa where relname = '{}' and attname = '{}' and pa.attrelid = pc.oid and pd.objoid = pc.oid and pd.objsubid = pa.attnum".format(
+            tab.name, self.name
+        )
+        c = tab.database().connector._execute(None, sql_cpt)  # Execute Check query
+        res = tab.database().connector._fetchone(c)[0]  # Store result
+        if res == 1:
+            # When a comment exists
+            c = tab.database().connector._execute(None, sql)  # Execute query
+            res = tab.database().connector._fetchone(c)[0]  # Store result
+            tab.database().connector._close_cursor(c)  # Close cursor
+            return res  # Return comment
+        else:
+            return ""
+
 
 class PGTableConstraint(TableConstraint):
 
     def __init__(self, row, table):
         TableConstraint.__init__(self, table)
-        self.name, constr_type_str, self.isDefferable, self.isDeffered, columns = row[:5]
-        self.columns = map(int, columns.split(' '))
+        self.name, constr_type_str, self.isDefferable, self.isDeffered, columns = row[
+            :5
+        ]
+        self.columns = list(map(int, columns.split(" ")))
 
         if constr_type_str in TableConstraint.types:
             self.type = TableConstraint.types[constr_type_str]
@@ -402,7 +569,7 @@ class PGTableIndex(TableIndex):
     def __init__(self, row, table):
         TableIndex.__init__(self, table)
         self.name, columns, self.isUnique = row
-        self.columns = map(int, columns.split(' '))
+        self.columns = list(map(int, columns.split(" ")))
 
 
 class PGTableTrigger(TableTrigger):
